/**
 * Copyright Marcus Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of LibDesQ (https://gitlab.com/DesQ/libdesq)
 * This library contains various  Core and Gui classes which are used
 * through out the DesQ Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#pragma once

/* For QString, QThread, QTimer, etc.. */
#include <QtCore>

#include <QLocalSocket>

namespace DesQ {
    namespace IPC {
        class Client;
    }
}

class DesQ::IPC::Client : public QLocalSocket {
    Q_OBJECT;

    public:
        enum ConnectionMode {
            Broadcast = 0xA6C765,           // Receive broadcasts from the server (Server -> Client) No acks
            Exchange,                       // Exchange information with the server (Server <-> Client) Both
                                            // sides ack
            Message,                        // Send commands to the server (Client -> Server ) Server acks
            Generic                         // For use with theDesQ::IPC::Server
        };

        Client( ConnectionMode mode, QObject *parent = nullptr );

        /*
         * Blocking call: Wait till the connection is made, send a register message and receive an ack
         * Broadcast: the server will ack the request, and save the connection for later messages
         * Communicate: the server will ack the request, and save the connection for later messages
         * If we're sending a command, the server waits for 100 ms for the command and then disconnects.
         */
        void connectToServer( const QString& serverName, QIODevice::OpenMode openMode = ReadWrite );

        /*
         * Blocking call:
         * Send a message to the server
         */
        bool sendMessage( QString );

        /* Send ack */
        void ack( QByteArray msg );

        /* Wait for ack */
        bool waitForAck( int timeout = -1 );

        /* Wait a predefined time for ready read and return the data read. */
        QByteArray readAll();

        /* Write and check of error. Disconnect on error */
        void write( QByteArray );

    private:
        ConnectionMode mCommMode;
        bool mConnected = false;
        bool mIsAcked   = false;
};
